// params.cc

#include "params.h"
#include <iostream>
#include <cstdlib>

#include <iostream>
#include <ctime>

namespace CommonTilton
{

 // Constructor
  Params::Params(const string& value)
  {
    version = value;

    mask_flag = false;
    mask_value = 0;
    mask_value_flag = false;
    bias_value = 0;
    fc_option = 1;
    edge_threshold = 0.0;
    output_type = 1;
    scale_output_flag = false;

    return;
  }

 // Destructor...
  Params::~Params() 
  {
    return;
  }

 // Print version
  void Params::print_version( )
  {
    cout << endl << version << endl << endl;

    return;
  }

 // Read parameters
  bool Params::read(const char *param_file)
  {
//    cout << "Reading parameters from " << param_file << endl;
    ifstream param_fs(param_file);
    if (!param_fs.is_open())
    {
      cout << "Cannot open input file " << param_file << endl;
      return false;
    }

  // Read initial parameters from parameter file until End-of-File reached
    bool input_image_flag = false;
    bool edge_operation_flag = false;
    bool output_image_flag = false;
    bool output_mask_image_flag = false;
    int sub_pos;
    string line, sub_string;
    while (!param_fs.eof())
    {
      getline(param_fs,line);
      sub_pos = line.find("-");
      while ((!param_fs.eof()) && (sub_pos != 0))
      {
        getline(param_fs,line);
        sub_pos = line.find("-");
      }
      if (line.find("-input_image") != string::npos)
      {
        input_image_file = process_line(line,false);
        input_image_flag = true;
      }
      if ((line.find("-mask") != string::npos) && (line.find("-mask_value") == string::npos))
      {
        mask_file = process_line(line,false);
        mask_flag = true;
      }
      if (line.find("-mask_value") != string::npos)
      {
        sub_string = process_line(line,false);
        mask_value = atoi(sub_string.c_str());
        mask_value_flag = true;
      }
      if (line.find("-bias_value") != string::npos)
      {
        sub_string = process_line(line,false);
        bias_value = atoi(sub_string.c_str());
      }
      if (line.find("-edge_operation") != string::npos)
      {
        sub_string = process_line(line,false);
        if (sub_string == "Prewitt")
        {
          edge_operation = Prewitt;
          edge_operation_flag = true;
        }
        else if (sub_string == "Sobel")
        {
          edge_operation = Sobel;
          edge_operation_flag = true;
        }
        else if (sub_string == "Scharr")
        {
          edge_operation = Scharr;
          edge_operation_flag = true;
        }
        else if (sub_string == "Frei-Chen")
        {
          edge_operation = Frei_Chen;
          edge_operation_flag = true;
        }
        else
        { 
          edge_operation_flag = false;
        }
      }
      if (line.find("-Frei-Chen_option") != string::npos)
      {
        sub_string = process_line(line,false);
        fc_option = atoi(sub_string.c_str());
      }
      if (line.find("-edge_threshold") != string::npos)
      {
        sub_string = process_line(line,false);
        edge_threshold = (float) atof(sub_string.c_str());
      }
      if (line.find("-output_type") != string::npos)
      {
        sub_string = process_line(line,false);
        output_type = atoi(sub_string.c_str());
      }
      if (line.find("-scale_output") != string::npos)
      {
        sub_string = process_line(line,false);
        sub_pos = atoi(sub_string.c_str());
        if (sub_pos == 1)
          scale_output_flag = true;
        else
          scale_output_flag = false;
      }
      if (line.find("-output_image") != string::npos)
      {
        output_image_file = process_line(line,false);
        output_image_flag = true;
      }
      if (line.find("-output_mask_image") != string::npos)
      {
        output_mask_image_file = process_line(line,false);
        output_mask_image_flag = true;
      }
    }

    param_fs.close();

// Exit with false status if a required parameter is not set, or an improper parameter
// setting is detected.
    if (!input_image_flag)
    {
      cout << "ERROR: -input_image (Input image file) is required" << endl;
      return false;
    }
    if (!edge_operation_flag)
    {
      cout << "ERROR: -edge_operation (Edge operation type) is required" << endl;
      return false;
    }
    if ((fc_option < 1) or (fc_option > 2))
    {
      cout << "ERROR: Invalid Frei-Chen_option. -Frei-Chen_option must be 1 or 2" << endl;
      return false;
    }
    if ((output_type < 1) or (output_type > 4))
    {
      cout << "ERROR: Invalid output_type (output type). -output_type must be 1, 2, 3 or 4" << endl;
      return false;
    }
    if (!output_image_flag)
    {
      cout << "ERROR: -output_image (Output image file) is required" << endl;
      return false;
    }
    if (!output_mask_image_flag)
    {
      cout << "ERROR: -output_mask_image (Output mask image file) is required" << endl;
      return false;
    }

    return true;
  }

 // Print parameters
  void Params::print()
  {
  // Print version
      cout << "This is " << version << endl << endl;

  // Print input parameters
      cout << "Input image file name: " << input_image_file << endl;
      if (mask_flag)
        cout << "Input image mask file name: " << mask_file << endl;
      if (mask_value_flag)
        cout << "Invalid data value in input image mask file = " << mask_value << endl;
      if (bias_value != 0)
        cout << "Bias value = " << bias_value << endl;
      switch (edge_operation)
      {
        case Prewitt:   cout << "Requested Prewitt edge operation" << endl;
                        break;
        case Sobel:     cout << "Requested Sobel edge operation" << endl;
                        break;
        case Scharr:    cout << "Requested Scharr edge operation" << endl;
                        break;
        case Frei_Chen: cout << "Requested Frei-Chen edge operation" << endl;
                        switch (fc_option)
                        {
                          case 1:  cout << "Requested edge Frei-Chen option" << endl;
                                   break;
                          case 2:  cout << "Requested line-edge Frei-Chen option" << endl;
                                   break;
                          default: cout << "WARNING: Requested invalid Frei-Chen option" << endl;
                                   break;
                        }
                        break;
        default:        cout << "WARNING: Requested invalid edge operation (Unknown)" << endl;
                        break;
      }
      if (edge_threshold > 0.0)
        cout << "Edge threshold = " << edge_threshold << endl;
      switch (output_type)
      {
        case 1:  cout << "Requested multispectral output" << endl;
                 break;
        case 2:  cout << "Requested band average output" << endl;
                 break;
        case 3:  cout << "Requested band maximum output" << endl;
                 break;
        case 4:  cout << "Requested band minimum output" << endl;
                 break;
        default: cout << "WARNING: Invalid output_type" << endl;
                 break;
      }
      if (scale_output_flag)
        cout << "Scaling of the edge values requested" << endl;
      cout << "Output image file name: " << output_image_file << endl;
      cout << "Output mask image file name: " << output_mask_image_file << endl;

      return;
  }

  string process_line(const string& line, const bool& list_flag)
  {
    int sub_pos, sub_pos_space, sub_pos_tab;
    string sub_string;

    sub_pos_space = line.find_first_of(" ");
    sub_pos_tab = line.find_first_of("\t");
    sub_pos = sub_pos_space;
    if (sub_pos ==  (int) string::npos)
      sub_pos = sub_pos_tab;
    if (sub_pos ==  (int) string::npos)
      return " ";

    sub_string = line.substr(sub_pos);
    while ((sub_string.substr(0,1) == "\t") || (sub_string.substr(0,1) == " "))
      sub_string = line.substr(++sub_pos);
#ifndef WINDOWS
    if (list_flag)
      return sub_string;

    sub_pos = sub_string.find_first_of(" ");
    if (sub_pos !=  (int) string::npos)
      sub_string = sub_string.substr(0,sub_pos);
    sub_pos = sub_string.find_first_of("\t");
    if (sub_pos !=  (int) string::npos)
      sub_string = sub_string.substr(0,sub_pos);
    sub_pos = sub_string.find_first_of("\r");
    if (sub_pos !=  (int) string::npos)
      sub_string = sub_string.substr(0,sub_pos);
#endif
    return sub_string;
  }

} // namespace CommonTilton
