/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose:  Provides the Params class, which is a class 
   >>>>                 holding program parameters.
   >>>>
   >>>>    Written By:  James C. Tilton, MC 606.3, NASA's GSFC, Greenbelt, MD 20771
   >>>>                 e-mail:  James.C.Tilton@nasa.gov
   >>>>
   >>>>          Date:  March 4, 2015
   >>>> Modifications:  March 6, 2015 - Added output_mask
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */
#ifndef PARAMS_H
#define PARAMS_H

#include <string>
#include <fstream>
#include <sstream>
#include <stdexcept>

using namespace std;

namespace CommonTilton
{
// Params class
  class Params 
  {
    public:
    // Constructor and Destructor
      Params(const string& value);
      virtual ~Params();

    // Member functions
      void print_version();
      bool read(const char *param_file);
      void print();

    // Member variables (public)
    /*-- First input image (required) --*/
      string first_image_file;     /*-- USER INPUT IMAGE FILENAME --*/

    /*-- Second input image (required) --*/
      string second_image_file;    /*-- USER INPUT IMAGE FILENAME --*/

    /*-- Input mask image (optional) --*/
      string input_mask_file;      /*-- USER INPUT IMAGE FILENAME --*/
      bool   input_mask_flag;      /*-- EXISTENCE FLAG --*/

    /*-- Output bdr image (required) --*/
      string bdr_image_file;       /*-- USER OUTPUT IMAGE FILENAME --*/

    /*-- Output mask image (required) --*/
      string output_mask_file;       /*-- USER OUTPUT IMAGE FILENAME --*/

    /* Program version */
      string version;              /* -- PROGRAM PARAMETER --*/

    // FRIEND FUNCTIONS and CLASSES
    protected:

    private:
  };

  string process_line(const string& line, const bool& list_flag);

} // CommonTilton

#endif /* PARAMS_H */
